==> llm-docs-0.1.5.tgz: package/docs/api/normalization.md <==
---
id: api-normalization
---
# API Normalization Guide

<!-- title: API Normalization Guide -->
<!-- description: Guide for API normalization and standardization in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the API normalization practices for the LLM Platform Platform. It covers the standardization of API responses, error handling, and data structures across all endpoints.

## Architecture

### Core Components

1. **Normalization Service**
   - Response transformer
   - Error normalizer
   - Schema validator
   - Cache manager

2. **Integration Points**
   - REST API
   - GraphQL API
   - WebSocket API
   - Event streams

3. **Validation Layer**
   - Schema validation
   - Type checking
   - Format validation
   - Security validation

## Implementation Guide

### Response Normalization

```typescript
// src/services/normalization.service.ts
export class NormalizationService {
  async normalize<T>(data: any, schema: Schema): Promise<NormalizedResponse<T>> {
    // Validate schema
    await this.validateSchema(data, schema);
    
    // Transform data
    const transformed = await this.transform(data);
    
    // Add metadata
    return this.addMetadata(transformed);
  }
  
  private async validateSchema(data: any, schema: Schema): Promise<void> {
    // Validate against schema
  }
  
  private async transform(data: any): Promise<any> {
    // Transform data
  }
  
  private addMetadata<T>(data: T): NormalizedResponse<T> {
    // Add metadata
  }
}
```

### Error Normalization

```typescript
// src/services/error-normalizer.service.ts
export class ErrorNormalizerService {
  normalize(error: any): NormalizedError {
    // Normalize error
    return {
      code: this.getErrorCode(error),
      message: this.getErrorMessage(error),
      details: this.getErrorDetails(error)
    };
  }
  
  private getErrorCode(error: any): string {
    // Get error code
  }
  
  private getErrorMessage(error: any): string {
    // Get error message
  }
  
  private getErrorDetails(error: any): any {
    // Get error details
  }
}
```

### Schema Validation

```typescript
// src/services/schema-validator.service.ts
export class SchemaValidatorService {
  async validate(data: any, schema: Schema): Promise<ValidationResult> {
    // Validate data
    const result = await this.validator.validate(data, schema);
    
    // Check for errors
    if (result.errors.length > 0) {
      throw new ValidationError(result.errors);
    }
    
    return result;
  }
}
```

## Integration Guide

### REST API Integration

```typescript
// src/controllers/base.controller.ts
export class BaseController {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/controllers/agent.controller.ts
@Controller('agents')
export class AgentController extends BaseController {
  @Get()
  async list(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

### GraphQL Integration

```typescript
// src/resolvers/base.resolver.ts
export class BaseResolver {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/resolvers/agent.resolver.ts
@Resolver('Agent')
export class AgentResolver extends BaseResolver {
  @Query()
  async agents(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

### WebSocket Integration

```typescript
// src/gateways/base.gateway.ts
export class BaseGateway {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/gateways/agent.gateway.ts
@WebSocketGateway()
export class AgentGateway extends BaseGateway {
  @SubscribeMessage('listAgents')
  async listAgents(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

## Best Practices

1. **Response Structure**
   - Use consistent format
   - Include metadata
   - Handle pagination
   - Support filtering

2. **Error Handling**
   - Normalize errors
   - Include details
   - Use proper codes
   - Log errors

3. **Data Validation**
   - Validate schemas
   - Check types
   - Format data
   - Sanitize input

4. **Performance**
   - Cache responses
   - Optimize transforms
   - Batch requests
   - Stream data

## Related Documentation

- [OpenAPI Validation](openapi-validation)
- [API Reference](reference)
- [REST API](rest)
- [GraphQL API](graphql)
- [Security Guide](../operations/security)
- [Development Guide](../development/README)

==> llm-docs-1.0.1.tgz: package/docs/api/normalization.md <==
---
id: api-normalization
---
# API Normalization Guide

<!-- title: API Normalization Guide -->
<!-- description: Guide for API normalization and standardization in the Bluefly Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the API normalization practices for the Bluefly Platform. It covers the standardization of API responses, error handling, and data structures across all endpoints.

## Architecture

### Core Components

1. **Normalization Service**
   - Response transformer
   - Error normalizer
   - Schema validator
   - Cache manager

2. **Integration Points**
   - REST API
   - GraphQL API
   - WebSocket API
   - Event streams

3. **Validation Layer**
   - Schema validation
   - Type checking
   - Format validation
   - Security validation

## Implementation Guide

### Response Normalization

```typescript
// src/services/normalization.service.ts
export class NormalizationService {
  async normalize<T>(data: any, schema: Schema): Promise<NormalizedResponse<T>> {
    // Validate schema
    await this.validateSchema(data, schema);
    
    // Transform data
    const transformed = await this.transform(data);
    
    // Add metadata
    return this.addMetadata(transformed);
  }
  
  private async validateSchema(data: any, schema: Schema): Promise<void> {
    // Validate against schema
  }
  
  private async transform(data: any): Promise<any> {
    // Transform data
  }
  
  private addMetadata<T>(data: T): NormalizedResponse<T> {
    // Add metadata
  }
}
```

### Error Normalization

```typescript
// src/services/error-normalizer.service.ts
export class ErrorNormalizerService {
  normalize(error: any): NormalizedError {
    // Normalize error
    return {
      code: this.getErrorCode(error),
      message: this.getErrorMessage(error),
      details: this.getErrorDetails(error)
    };
  }
  
  private getErrorCode(error: any): string {
    // Get error code
  }
  
  private getErrorMessage(error: any): string {
    // Get error message
  }
  
  private getErrorDetails(error: any): any {
    // Get error details
  }
}
```

### Schema Validation

```typescript
// src/services/schema-validator.service.ts
export class SchemaValidatorService {
  async validate(data: any, schema: Schema): Promise<ValidationResult> {
    // Validate data
    const result = await this.validator.validate(data, schema);
    
    // Check for errors
    if (result.errors.length > 0) {
      throw new ValidationError(result.errors);
    }
    
    return result;
  }
}
```

## Integration Guide

### REST API Integration

```typescript
// src/controllers/base.controller.ts
export class BaseController {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/controllers/agent.controller.ts
@Controller('agents')
export class AgentController extends BaseController {
  @Get()
  async list(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

### GraphQL Integration

```typescript
// src/resolvers/base.resolver.ts
export class BaseResolver {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/resolvers/agent.resolver.ts
@Resolver('Agent')
export class AgentResolver extends BaseResolver {
  @Query()
  async agents(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

### WebSocket Integration

```typescript
// src/gateways/base.gateway.ts
export class BaseGateway {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async normalizeResponse<T>(
    data: any,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      return await this.normalizationService.normalize(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
}

// src/gateways/agent.gateway.ts
@WebSocketGateway()
export class AgentGateway extends BaseGateway {
  @SubscribeMessage('listAgents')
  async listAgents(): Promise<NormalizedResponse<Agent[]>> {
    const agents = await this.agentService.list();
    return this.normalizeResponse(agents, agentListSchema);
  }
}
```

## Best Practices

1. **Response Structure**
   - Use consistent format
   - Include metadata
   - Handle pagination
   - Support filtering

2. **Error Handling**
   - Normalize errors
   - Include details
   - Use proper codes
   - Log errors

3. **Data Validation**
   - Validate schemas
   - Check types
   - Format data
   - Sanitize input

4. **Performance**
   - Cache responses
   - Optimize transforms
   - Batch requests
   - Stream data

## Related Documentation

- [OpenAPI Validation](openapi-validation)
- [API Reference](reference)
- [REST API](rest)
- [GraphQL API](graphql)
- [Security Guide](../operations/security)
- [Development Guide](../development/README)
